#
# DON'T CHANGE THIS FILE MANUALLY
#
# This file is generated by:
# .../ha-services/dev-cli.py update-ha-data
#
# Last Update: 2025-09-25T18:22:29.675522
#
# Generated with data from Home Assistant package:
#
# homeassistant v2025.9.4
#

HA_SENSOR_DATA = [
    {'device_class': 'absolute_humidity', 'state_classes': {'measurement'}, 'units': {'mg/m³', 'g/m³'}},
    {'device_class': 'apparent_power', 'state_classes': {'measurement'}, 'units': {'mVA', 'VA'}},
    {'device_class': 'aqi', 'state_classes': {'measurement'}, 'units': set()},
    {
        'device_class': 'area',
        'state_classes': {'measurement_angle', 'total', 'measurement', 'total_increasing'},
        'units': {'in²', 'mm²', 'yd²', 'ha', 'm²', 'km²', 'mi²', 'cm²', 'ac', 'ft²'},
    },
    {
        'device_class': 'atmospheric_pressure',
        'state_classes': {'measurement'},
        'units': {'mbar', 'psi', 'cbar', 'hPa', 'mmHg', 'inHg', 'kPa', 'bar', 'Pa'},
    },
    {'device_class': 'battery', 'state_classes': {'measurement'}, 'units': {'%'}},
    {'device_class': 'blood_glucose_concentration', 'state_classes': {'measurement'}, 'units': {'mg/dL', 'mmol/L'}},
    {'device_class': 'carbon_monoxide', 'state_classes': {'measurement'}, 'units': {'ppm'}},
    {'device_class': 'carbon_dioxide', 'state_classes': {'measurement'}, 'units': {'ppm'}},
    {'device_class': 'conductivity', 'state_classes': {'measurement'}, 'units': {'S/cm', 'μS/cm', 'mS/cm'}},
    {'device_class': 'current', 'state_classes': {'measurement'}, 'units': {'A', 'mA'}},
    {
        'device_class': 'data_rate',
        'state_classes': {'measurement'},
        'units': {'KiB/s', 'B/s', 'kB/s', 'MB/s', 'kbit/s', 'GiB/s', 'GB/s', 'MiB/s', 'Mbit/s', 'bit/s', 'Gbit/s'},
    },
    {
        'device_class': 'data_size',
        'state_classes': {'measurement_angle', 'total', 'measurement', 'total_increasing'},
        'units': {
            'TB',
            'EB',
            'PB',
            'bit',
            'YiB',
            'kbit',
            'ZiB',
            'KiB',
            'kB',
            'MiB',
            'Mbit',
            'Gbit',
            'ZB',
            'TiB',
            'PiB',
            'EiB',
            'YB',
            'GB',
            'GiB',
            'MB',
            'B',
        },
    },
    {
        'device_class': 'distance',
        'state_classes': {'measurement_angle', 'total', 'measurement', 'total_increasing'},
        'units': {'cm', 'm', 'ft', 'in', 'nmi', 'km', 'mm', 'yd', 'mi'},
    },
    {
        'device_class': 'duration',
        'state_classes': {'measurement_angle', 'total', 'measurement', 'total_increasing'},
        'units': {'min', 'h', 's', 'μs', 'd', 'ms'},
    },
    {
        'device_class': 'energy',
        'state_classes': {'total', 'total_increasing'},
        'units': {'Gcal', 'mWh', 'Wh', 'kWh', 'J', 'Mcal', 'kJ', 'MWh', 'kcal', 'TWh', 'GJ', 'MJ', 'GWh', 'cal'},
    },
    {
        'device_class': 'energy_distance',
        'state_classes': {'measurement'},
        'units': {'km/kWh', 'mi/kWh', 'kWh/100km', 'Wh/km'},
    },
    {
        'device_class': 'energy_storage',
        'state_classes': {'measurement'},
        'units': {'Gcal', 'mWh', 'Wh', 'kWh', 'J', 'Mcal', 'kJ', 'MWh', 'kcal', 'TWh', 'GJ', 'MJ', 'GWh', 'cal'},
    },
    {'device_class': 'frequency', 'state_classes': {'measurement'}, 'units': {'MHz', 'kHz', 'Hz', 'GHz'}},
    {'device_class': 'gas', 'state_classes': {'total', 'total_increasing'}, 'units': {'CCF', 'L', 'm³', 'ft³'}},
    {'device_class': 'humidity', 'state_classes': {'measurement'}, 'units': {'%'}},
    {'device_class': 'illuminance', 'state_classes': {'measurement'}, 'units': {'lx'}},
    {'device_class': 'irradiance', 'state_classes': {'measurement'}, 'units': {'W/m²', 'BTU/(h⋅ft²)'}},
    {'device_class': 'moisture', 'state_classes': {'measurement'}, 'units': {'%'}},
    {'device_class': 'nitrogen_dioxide', 'state_classes': {'measurement'}, 'units': {'μg/m³'}},
    {'device_class': 'nitrogen_monoxide', 'state_classes': {'measurement'}, 'units': {'μg/m³'}},
    {'device_class': 'nitrous_oxide', 'state_classes': {'measurement'}, 'units': {'μg/m³'}},
    {'device_class': 'ozone', 'state_classes': {'measurement'}, 'units': {'μg/m³'}},
    {'device_class': 'ph', 'state_classes': {'measurement'}, 'units': set()},
    {'device_class': 'pm1', 'state_classes': {'measurement'}, 'units': {'μg/m³'}},
    {'device_class': 'pm10', 'state_classes': {'measurement'}, 'units': {'μg/m³'}},
    {'device_class': 'pm25', 'state_classes': {'measurement'}, 'units': {'μg/m³'}},
    {'device_class': 'power_factor', 'state_classes': {'measurement'}, 'units': {'%'}},
    {'device_class': 'power', 'state_classes': {'measurement'}, 'units': {'MW', 'TW', 'GW', 'mW', 'W', 'kW'}},
    {
        'device_class': 'precipitation',
        'state_classes': {'measurement_angle', 'total', 'measurement', 'total_increasing'},
        'units': {'in', 'cm', 'mm'},
    },
    {
        'device_class': 'precipitation_intensity',
        'state_classes': {'measurement'},
        'units': {'in/h', 'in/d', 'mm/d', 'mm/h'},
    },
    {
        'device_class': 'pressure',
        'state_classes': {'measurement'},
        'units': {'mbar', 'psi', 'cbar', 'hPa', 'mmHg', 'inHg', 'kPa', 'bar', 'Pa'},
    },
    {'device_class': 'reactive_energy', 'state_classes': {'total', 'total_increasing'}, 'units': {'varh', 'kvarh'}},
    {'device_class': 'reactive_power', 'state_classes': {'measurement'}, 'units': {'mvar', 'kvar', 'var'}},
    {'device_class': 'signal_strength', 'state_classes': {'measurement'}, 'units': {'dB', 'dBm'}},
    {'device_class': 'sound_pressure', 'state_classes': {'measurement'}, 'units': {'dBA', 'dB'}},
    {
        'device_class': 'speed',
        'state_classes': {'measurement'},
        'units': {'mph', 'mm/s', 'Beaufort', 'mm/h', 'ft/s', 'm/s', 'mm/d', 'km/h', 'in/s', 'in/h', 'in/d', 'kn'},
    },
    {'device_class': 'sulphur_dioxide', 'state_classes': {'measurement'}, 'units': {'μg/m³'}},
    {'device_class': 'temperature', 'state_classes': {'measurement'}, 'units': {'K', '°F', '°C'}},
    {'device_class': 'volatile_organic_compounds', 'state_classes': {'measurement'}, 'units': {'mg/m³', 'μg/m³'}},
    {'device_class': 'volatile_organic_compounds_parts', 'state_classes': {'measurement'}, 'units': {'ppb', 'ppm'}},
    {'device_class': 'voltage', 'state_classes': {'measurement'}, 'units': {'MV', 'kV', 'μV', 'mV', 'V'}},
    {
        'device_class': 'volume',
        'state_classes': {'total', 'total_increasing'},
        'units': {'mL', 'fl. oz.', 'ft³', 'gal', 'm³', 'CCF', 'L'},
    },
    {
        'device_class': 'volume_flow_rate',
        'state_classes': {'measurement'},
        'units': {'L/h', 'm³/h', 'gal/min', 'm³/min', 'ft³/min', 'L/min', 'm³/s', 'mL/s', 'L/s'},
    },
    {
        'device_class': 'volume_storage',
        'state_classes': {'measurement'},
        'units': {'mL', 'fl. oz.', 'ft³', 'gal', 'm³', 'CCF', 'L'},
    },
    {
        'device_class': 'water',
        'state_classes': {'total', 'total_increasing'},
        'units': {'ft³', 'gal', 'm³', 'CCF', 'L'},
    },
    {
        'device_class': 'weight',
        'state_classes': {'total', 'measurement', 'total_increasing'},
        'units': {'lb', 'kg', 'oz', 'mg', 'st', 'g', 'μg'},
    },
    {'device_class': 'wind_direction', 'state_classes': {'measurement_angle'}, 'units': {'°'}},
    {
        'device_class': 'wind_speed',
        'state_classes': {'measurement'},
        'units': {'mph', 'mm/s', 'Beaufort', 'ft/s', 'm/s', 'km/h', 'in/s', 'kn'},
    },
]
